{==============================================================================
        :Unit: ADODBNavigator.pas
      :Author: Emil Lefkof, Consultant
                 Melman Consulting Group, Copyright 2000.
:Date Created: 01/08/2000
 :Description: This component is just an inhertited DBNavigator that overrides
               the button click to call ADODataset.Requery instead of
               ADODataSet.Refresh because refresh behind the scenes calls a
               Resync method which is horribly slow for ADOTables, while the
               Requery method is quite fast.
============================================================================== }
unit ADODBNavigator;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, DBCtrls, ADODB, Db;

type
  TADODBNavigator = class(TDBNavigator)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    procedure BtnClick(Index: TNavigateBtn); override;
  published
    { Published declarations }
  end;

procedure Register;

implementation

procedure TADODBNavigator.BtnClick(Index: TNavigateBtn);
begin
  if (DataSource <> nil) and (DataSource.State <> dsInactive) then
  begin
    if not (csDesigning in ComponentState) and Assigned(BeforeAction) then
      BeforeAction(Self, Index);
    with TADODataset(DataSource.DataSet) do
    begin
      case Index of
        nbPrior: Prior;
        nbNext: Next;
        nbFirst: First;
        nbLast: Last;
        nbInsert: Insert;
        nbEdit: Edit;
        nbCancel: Cancel;
        nbPost: Post;
        nbRefresh: Requery; //call requery instead of refresh
        nbDelete:
          if not ConfirmDelete or
            (MessageDlg('Delete record?', mtConfirmation,
            mbOKCancel, 0) <> idCancel) then Delete;
      end;
    end;
  end;
  if not (csDesigning in ComponentState) and Assigned(OnClick) then
    OnClick(Self, Index);
end;

procedure Register;
begin
  RegisterComponents('Data Controls', [TADODBNavigator]);
end;

end.
