unit BrowsVw;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Grids, Outline, ExtCtrls;

type
  TBrowseView = class(TForm)
    pnlBottom: TPanel;
	 pnlButton: TPanel;
	 btnClose: TButton;
	 pnlList: TPanel;
	 theOutline: TOutline;
	 btnExpand: TButton;
	 btnAbout: TButton;
	 procedure FormCreate(Sender: TObject);
	 procedure btnExpandClick(Sender: TObject);
	 procedure btnCloseClick(Sender: TObject);
	 procedure btnAboutClick(Sender: TObject);
  private
    { Private declarations }
  public
	 { Public declarations }
  end;

var
  BrowseView: TBrowseView;

implementation

uses
	AboutBox, DB, DbiTypes, DbiProcs;

{$R *.DFM}


{----------------------------------------------------------------------------
	Fill the outline when the form is created
----------------------------------------------------------------------------}
function RecurseCfgInfo( theOutline: TOutline; Path, Node: PChar; Level: Word ) : Word;
var
	Cursor: HDBICur;
	SPath: array[0..63] of Char;
	ConfigDesc: CFGDesc;
	I: Word;
begin
	{ If this is the first or second level of recursion }
	if Path = nil then
		{ Build the root '\' or first level '\PATH' path name }
		StrLFmt( SPath, SizeOf(SPath) - 1, '\%s', [Node] )
	else
		{Otherwise, we can simple append the new node name '\PATH\SUB' }
		StrLFmt( SPath, SizeOf(SPath) - 1, '%s\%s', [Path,Node] );
	{ Open a cursor on the newly created path }
	Check(DbiOpenCfgInfoList( nil, dbiREADONLY, cfgPERSISTENT, SPath, Cursor ));
	{ Track the AddChild for the most previous call to RecurseCfgInfo }
	I := 0;
	try
		{ For each record in our new cursor }
		while DbiGetNextRecord( Cursor, dbiNOLOCK, @ConfigDesc, nil ) = 0 do
			with ConfigDesc do begin
				Inc( I );
				{ If the record / node has levels below }
				if bHasSubnodes then begin
					{ First add the node to the outline }
					theOutline.AddChild( Level, StrPas( szNodeName ) );
					{ And recursively call to handle the lower levels }
					if Level = 0 then
						Inc( I, RecurseCfgInfo( theOutline, nil, szNodeName, Level+I ) )
					else
						Inc( I, RecurseCfgInfo( theOutline, SPath, szNodeName, Level+I ) );
				{ Otherwise }
				end else begin
					{ Convert the value to ANSI; copy from the VCL source }
					OemToAnsi( szValue, szValue );
					{ And add it to the outline }
					theOutline.AddChild( Level, Format('%s=%s', [szNodeName, szValue]) );
				end;
			end;
	finally
		DbiCloseCursor( Cursor );
	end;
	{ Return the index of the last item this call added to the outline }
	Result := I;
end;


{----------------------------------------------------------------------------
	Fill the outline when the form is created
----------------------------------------------------------------------------}
procedure TBrowseView.FormCreate(Sender: TObject);
begin
	RecurseCfgInfo( theOutline, nil, '', 0 );
end;


{----------------------------------------------------------------------------
	Expand / collapse the highlighted node
----------------------------------------------------------------------------}
procedure TBrowseView.btnExpandClick(Sender: TObject);
begin
	with theOutline do
		if SelectedItem <> 0 then
			with Items[SelectedItem] do
				Expanded := not Expanded;
end;


{----------------------------------------------------------------------------
	Handle the close button
----------------------------------------------------------------------------}
procedure TBrowseView.btnCloseClick(Sender: TObject);
begin
	Close;
end;


{----------------------------------------------------------------------------
	User wants the about box
----------------------------------------------------------------------------}
procedure TBrowseView.btnAboutClick(Sender: TObject);
var
	theAbout: TAbout;
begin
	theAbout := TAbout.Create( Self );
	try
		theAbout.ShowModal;
	finally
		theAbout.Free;
	end;
end;


end.


