unit Conform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TConsoleForm = class(TForm)
    RunButton: TButton;
    Static: TStaticText;
    procedure RunButtonClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  ConsoleForm: TConsoleForm;

implementation
{This unit demonstrates a GUI application spawning a
console application, and capturing the output of the
console application to display in the GUI application.
The console app should be win32, I think

Any matters arising, questions, comments etc... contact

Martin Lafferty

robots@enterprise.net

Production Robots Engineering Ltd

}

function ExecConsoleApp(CommandLine: String;
                       OutputWindow: hWnd): Integer;
const
  BufSize = $4000;
  {for this simple example, the pipe should be able to
  hold the entire output of the console app.
  Predictably, this call hangs if CommandLine points to a
  Win32 GUI app. Tested with Win32 and MS-DOS console apps}
var
  StartupInfo:TStartupInfo;
  ProcessInfo:TProcessInformation;
  WriteHandle, ReadHandle: THandle;
  ReadBuf: array[0..BufSize] of Char;
  BytesRead: Integer;

begin
  FillChar(StartupInfo,SizeOf(StartupInfo), 0);
  FillChar(ReadBuf, SizeOf(ReadBuf), 0);
  CreatePipe(ReadHandle, WriteHandle, nil, BufSize);
  with StartupInfo do
  begin
    cb:= SizeOf(StartupInfo);
    dwFlags:= STARTF_USESHOWWINDOW or
               STARTF_USESTDHANDLES;
    hStdOutput:= WriteHandle;
    hStdError:= WriteHandle;
    wShowWindow:= SW_HIDE;
  end;
  if not CreateProcess(nil, PChar(CommandLine), nil, nil,
     false, DETACHED_PROCESS or NORMAL_PRIORITY_CLASS,
     nil, nil, StartupInfo, ProcessInfo) then
  begin
    Result:= -1
  end else
  begin
    WaitForSingleObject(ProcessInfo.hProcess, INFINITE);
    {if you want to do a running update of the output of
    the console app, you would not block here. Instead you
    could loop while GetExitCodeProcess returned STILL_ACTIVE
    all the while calling ReadFile on the pipe. Overlapped IO
    would be nice, but I am not sure it would work on an
    anonymous pipe. I have tried polling the pipe (works OK)
    but not overlapped IO. MGL}
    ReadFile( ReadHandle, ReadBuf, BufSize, BytesRead, nil);
    SetWindowText(OutputWindow, ReadBuf);
    GetExitCodeProcess(ProcessInfo.hProcess, Result);
    CloseHandle(ProcessInfo.hProcess);
    CloseHandle(ProcessInfo.hThread)
  end;
  CloseHandle(WriteHandle)
end;

const
  homedir = 'c:\delphi32\prel\';
  brcc = 'c:\delphi32\bin\brcc32';
  dcc = 'c:\delphi32\bin\dcc32';

{$R *.DFM}

procedure TConsoleForm.RunButtonClick(Sender: TObject);
var
  s: String;
  CAExitCode: Integer;
begin
  s:= dcc + ' ' + homedir + 'prel.dpk /b /q';
  if InputQuery('Execute Console App', 'Enter command line', s) then
  begin
    Static.Caption:= 'Executing ' + s;
    CAExitCode:= ExecConsoleApp(s, Static.Handle);
    if CAExitCode <> 0 then
    begin
      if CAExitCode = -1 then
        MessageDlg(Format('could not execute ''%s''', [s]),
                   mtError, [mbOK], 0)
      else
        MessageDlg(Format('%s returned exitcode of %d',
                 [s, CAExitCode]), mtError, [mbOK], 0)
    end
  end
end;

end.
