unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  BjTypes, StdCtrls, Spin, Menus, ComCtrls, Buttons, ExtCtrls;

type
  TMainForm = class(TForm)
    RunBut: TButton;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    CashLabel: TLabel;
    PlayedLabel: TLabel;
    Label3: TLabel;
    AvgLabel: TLabel;
    Label4: TLabel;
    Label2: TLabel;
    SpeedLabel: TLabel;
    Label5: TLabel;
    TimeLabel: TLabel;
    Label6: TLabel;
    ShufLabel: TLabel;
    Label8: TLabel;
    RangeLabel: TLabel;
    CloseBut: TButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    Help1: TMenuItem;
    About1: TMenuItem;
    View1: TMenuItem;
    CurrentGame1: TMenuItem;
    CardCountStats1: TMenuItem;
    Deck1: TMenuItem;
    OptBut: TButton;
    Print1: TMenuItem;
    N1: TMenuItem;
    procedure RunButClick(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure CloseButClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure OptButClick(Sender: TObject);
    procedure CardCountStats1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    Deck    : TDeck;
    procedure CreateParams(var Params: TCreateParams); override;
  end;

var
  MainForm: TMainForm;

implementation

uses DebugFm, About, Options, CountStat;

{$R *.DFM}
{$R sol.res}

const
   CONF95   = 1.959961;
   CONF99   = 2.575835;
   CONF999  = 3.290479;

function StandardDev(N: integer; XSum, X2Sum: double): double;
var
   Avg : double;
   tmp : double;
begin
   Avg := XSum / N;
   tmp := X2Sum - N * Avg*Avg;
   Result := Sqrt(tmp / N);
end;

procedure TMainForm.RunButClick(Sender: TObject);
label
   Win, Lose, Tie, Next;
var
   DealerHand  : THand;
   Player1     : TPlayer;
   Game        : integer;
   NumShuf     : integer;
   Winnings    : double;
   WinSum      : double;      // Sum of Winnings
   Win2Sum     : double;      // Sum of Winnings^2
   StDev, Range: double;
   t1, t2      : integer;
   tdif        : double;
   NumGame     : integer;
   MinBet      : integer;
   MaxBet      : integer;
   c, Bet      : double;
   i, cr       : integer;
   tc          : array[-30..30] of integer;
   tw          : array[-30..30] of double;
function WagerCount(count: double): double;
begin
   Result := MinBet;
   if count > 3.5 then Result := (Trunc(count)-1) * MinBet * 4;
   if Result > MaxBet then Result := MaxBet;
end;
begin
   for i := -30 to 30 do begin
      tc[i] := 0;
      tw[i] := 0;
   end;

   Randomize;
   NumGame := OptionsForm.NumGame.Value;
   Deck := TDeck.Create;
   Deck.NumDecks := OptionsForm.NumDeck.Position;
   Deck.Penetration := OptionsForm.DeckPenet.Value / 100.0;
   MinBet := OptionsForm.MinBet.Value;
   MaxBet := OptionsForm.MaxBet.Value;

   DealerHand := THand.Create;
   Player1    := TPlayer.Create;

   Player1.Cash := 0;
   WinSum := 0;
   Win2Sum := 0;
   Deck.Shuffle;
   NumShuf := 1;

   Screen.Cursor := crHourGlass;
try
   t1 := GetTickCount;
   for Game := 1 to NumGame do begin
      // Shuffle if needed
      if Deck.NeedShuffle then begin
         {$IFDEF Debug}
         Debug('');
         Debug('** Shuffling **');
         Debug('');
         {$ENDIF}
         Deck.Shuffle;
         NumShuf := NumShuf+1;
      end;

      DealerHand.InitHand;

      // Here's the card counting code
      c := Deck.TrueCount;
      bet := WagerCount(c);

      // if c < 1 then bet := 0;                         // Wong it baby!

      Player1.InitHands(bet);

      // True Count Stats
      cr := Round(c);
      if cr < -30 then cr := -30;
      if cr > 30 then cr := 30;
      tc[cr] := tc[cr] + 1;

      // Deal the cards
      Player1.Hands[0].AddCard(Deck.DealCard);
      DealerHand.AddCard(Deck.DealCard);
      Player1.Hands[0].AddCard(Deck.DealCard);
      DealerHand.AddCard(Deck.DealCard);

      {$IFDEF Debug}
      Debug('');
      Debug('Game # = ' + IntToStr(Game));
      Debug(Format('True Count = %1.1n   Wager = $%1.2n', [c, bet]));
      Debug('Dealer Hand = ' + DealerHand.Out);
      Debug('Player Hand = ' + Player1.Hands[0].Out);
      Debug('Player Play:');
      {$ENDIF}

      Player1.PlayHands(DealerHand.ShowCard, Deck);

      // Dealer doesn't play his if all players bust
      if not Player1.AllHandsBusted then begin
         {$IFDEF Debug}
         Debug('Dealer Play:');
         {$ENDIF}
         // Now the dealers turn
         while DealerHand.Total < 17 do
            DealerHand.TakeHit(Deck);
      end;

      Winnings := Player1.EvaluateAllHands(DealerHand);
      tw[cr] := tw[cr] + Winnings;
      WinSum := WinSum + Winnings;
      Win2Sum := Win2Sum + Winnings*Winnings;
      Player1.Cash := Player1.Cash + Winnings;
      {$IFDEF Debug}
      Debug(Format('Player Cash = $%1.0n', [Player1.Cash]));
      {$ENDIF}

      if ((Game and 4095) = 0) or (Game=NumGame) then begin
         CashLabel.Caption := Format('$%1.2n', [Player1.Cash]);
         AvgLabel.Caption := Format('$%1.3n', [Player1.Cash / Game]);
         PlayedLabel.Caption := Format('%1.0n of %1.0n', [Game+0.0, NumGame+0.0]);

         t2 := GetTickCount;
         tdif := (t2-t1) / 1000.0;
         SpeedLabel.Caption := Format('%6.0n', [game/tdif]);
         TimeLabel.Caption := Format('%3.1n', [tdif]);
         ShufLabel.Caption := Format('%1.0n', [NumShuf+0.0]);

         StDev := StandardDev(Game, WinSum, Win2Sum);
         Range := CONF95 * StDev / Sqrt(Game);
         RangeLabel.Caption := Format('%1.3n', [Range]);

         Update;
      end;
   end; // Game
finally
   Screen.Cursor := crDefault;
   Deck.Free;
   Player1.Free;
   DealerHand.Free;
end;

   with CountStatForm.Grid do begin
      for i := -30 to 30 do begin
         //Memo1.Lines.Add(Format('%8d  %8d  $%8.0n   Wager = $%4.0n',
         //[i, tc[i], tw[i], WagerCount(i)]));}
         Cells[1, i+31] := Format('%d', [tc[i]]);
         Cells[2, i+31] := Format('$%1.0n', [tw[i]]);
         if tc[i]<>0 then
            Cells[3, i+31] := Format('$%1.2n', [tw[i] / tc[i]])
         else
            Cells[3, i+31] := '';
         Cells[4, i+31] := Format('$%1.0n', [WagerCount(i)]);
      end;
   end;

end;

procedure TMainForm.Exit1Click(Sender: TObject);
begin
   Close;
end;

procedure TMainForm.CloseButClick(Sender: TObject);
begin
   Close;
end;

procedure TMainForm.About1Click(Sender: TObject);
begin
   AboutForm.ShowModal;
end;

procedure TMainForm.OptButClick(Sender: TObject);
begin
   OptionsForm.ShowModal;
end;

procedure TMainForm.CardCountStats1Click(Sender: TObject);
begin
   CountStatForm.Show;
end;

procedure TMainForm.CreateParams(var Params: TCreateParams);
begin
   inherited;
   Params.WndParent := 0;
end;

procedure TMainForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   Application.Terminate;
end;

procedure TMainForm.FormActivate(Sender: TObject);
begin
   ShowWindow(Application.Handle, SW_HIDE);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
   Application.OnActivate := FormActivate;
end;

end.
