(******************************************************************************
**
**  VersLook.Pas
**
*******************************************************************************)
unit Versunit;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Customs;

type
  TVersResourceReader = class(TForm)
    ListBox1: TListBox;
    BitBtn1: TBitBtn;
    OpenDialog1: TOpenDialog;
    Header1: THeader;
    Panel1: TPanel;
    ListBox2: TListBox;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    procedure BitBtn1Click(Sender: TObject);
    procedure Header1Sized(Sender: TObject; ASection, AWidth: Integer);
    procedure ListBox1Click(Sender: TObject);
    procedure ListBox2Click(Sender: TObject);
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure BitBtn3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure FillInResource(FileName: TFileName);
  public
    { Public declarations }
  end;

var
  VersResourceReader: TVersResourceReader;

implementation

{$R *.DFM}

uses
  WExe_Ver, VLookAbt;


(*
**  Procedure to create the Version resource object, and fill in the
**  two ListBoxes.
*)
procedure TVersResourceReader.FillInResource(FileName: TFileName);
var
  VersRes: TWinExeDllVer;
begin
  VersRes := TWinExeDllVer.CreateWithFile(FileName);
  try
    if not VersRes.VersionExists then
    begin
      MessageDlg('File does not contain a Version resource',
        mtInformation, [mbOk], 0);
      exit;
    end;
    ListBox1.Clear;
    ListBox2.Clear;
    with ListBox1.Items, Versres do
    begin
      Add('Comments:');                 ListBox2.Items.Add(Read(vtComments));
      Add('Company Name:');             ListBox2.Items.Add(Read(vtCompanyName));
      Add('File Description:');         ListBox2.Items.Add(Read(vtFileDescription));
      Add('File Version:');             ListBox2.Items.Add(Read(vtFileVersion));
      Add('Internal Name:');            ListBox2.Items.Add(Read(vtInternalName));
      Add('Legal Copyright:');          ListBox2.Items.Add(Read(vtLegalCopyright));
      Add('Legal Trade Marks:');        ListBox2.Items.Add(Read(vtLegalTradeMarks));
      Add('Original File Name:');       ListBox2.Items.Add(Read(vtOriginalFileName));
      Add('Private Build:');            ListBox2.Items.Add(Read(vtPrivateBuild));
      Add('Product Name:');             ListBox2.Items.Add(Read(vtProductName));
      Add('Product Version:');          ListBox2.Items.Add(Read(vtProductVersion));
      Add('Special Build:');            ListBox2.Items.Add(Read(vtSpecialBuild));
      Add('Signature:');                ListBox2.Items.Add(Read(vtSignature));
      Add('Struct Version:');           ListBox2.Items.Add(Read(vtStructVersion));
      Add('Internal File Verison:');    ListBox2.Items.Add(Read(vtFileVersionInt));
      Add('Internal Product Verison:'); ListBox2.Items.Add(Read(vtProductVersionInt));
      Add('File OS:');                  ListBox2.Items.Add(Read(vtFileOS));
      Add('File Type:');                ListBox2.Items.Add(Read(vtFileType));
      Add('File Sub Type:');            ListBox2.Items.Add(Read(vtFileSubType));
      Add('File Date:');                ListBox2.Items.Add(Read(vtFileDate));
      Add('File Flags:');               ListBox2.Items.Add(Read(vtFileFlags));
    end;
    Caption := 'Version Resource Viewer: ' + ExtractFileName(FileName);
  finally
    VersRes.Free;
  end;
end;

(*
**  Allow user to select the file to inspect, and pass name to Version object
*)
procedure TVersResourceReader.BitBtn1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
    FillInResource(OpenDialog1.FileName);
end;

(*
**  Ensure the Left list box width remains consistent with the header left
**  section width
*)
procedure TVersResourceReader.Header1Sized(Sender: TObject; ASection, AWidth: Integer);
begin
  ListBox1.Width := AWidth + 1;
end;

(*
**  Ensure that the two select bars stay in sync
*)
procedure TVersResourceReader.ListBox1Click(Sender: TObject);
begin
  ListBox2.ItemIndex := ListBox1.ItemIndex;
  ListBox2.TopIndex := ListBox1.TopIndex;
end;

(*
**  Ensure that the two select bars stay in sync
*)
procedure TVersResourceReader.ListBox2Click(Sender: TObject);
begin
  ListBox1.ItemIndex := ListBox2.ItemIndex;
  ListBox1.TopIndex := ListBox2.TopIndex;
end;

(*
**  Draw of listbox1 procedure to make text right justified
*)
procedure TVersResourceReader.ListBox1DrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  Text: array[0..255] of char;
  CurColour: TColor;
begin
  with (Control as TListBox).Canvas do
  begin
    FillRect(Rect);	                        { Clear old contents of rectangle }
    CurColour := Font.Color;                { Save existing colour }
    if (odDisabled in State) then
      Font.Color := clSilver                { Disabled writing colour }
    else if (odSelected in State) then
      Font.Color := clHighLightText         { Highlight bar colour }
    else
      Font.Color := clWindowText;           { Normal writing colour }

    DrawText(Handle, StrPCopy(Text,
      (Control as TListBox).Items.Strings[Index]),
      -1, Rect, DT_RIGHT or DT_NOCLIP or
      DT_SINGLELINE or DT_VCENTER);         { draw on the control canvas }
    Font.Color := CurColour;                { Restore Fint Colour }
  end;
end;

(*
**  Show about box
*)
procedure TVersResourceReader.BitBtn3Click(Sender: TObject);
var
  AbtBox: TLookVersAbout;
begin
  AbtBox := TLookVersAbout.Create(Self);
  try
    AbtBox.ShowModal;
  finally
    AbtBox.Free;
  end;
end;

(*
**  If a file name is passed on the command line, load up the resource
**  information automatically
*)
procedure TVersResourceReader.FormCreate(Sender: TObject);
begin
  if ParamStr(1) <> '' then                 { Parameter passed? }
    if FileExists(ParamStr(1)) then         { Valid filename? }
      FillInResource(ParamStr(1));          { Load in resource }
end;

end.
