unit main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ImgList, ToolWin, StdCtrls, SetupStructs, Options, Menus, IniFiles;

type
  TfrmMain = class(TForm)
    lvFiles: TListView;
    Label1: TLabel;
    dlgOpen: TOpenDialog;
    ToolBar1: TToolBar;
    ImageList1: TImageList;
    ToolButton1: TToolButton;
    ToolButton2: TToolButton;
    ToolButton3: TToolButton;
    ToolButton4: TToolButton;
    ToolButton5: TToolButton;
    ToolButton6: TToolButton;
    edSetupFile: TEdit;
    Button1: TButton;
    lblResult: TLabel;
    Progress: TProgressBar;
    ToolButton7: TToolButton;
    ToolButton8: TToolButton;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    N1: TMenuItem;
    Save1: TMenuItem;
    N2: TMenuItem;
    Exit1: TMenuItem;
    dlgSave: TSaveDialog;
    Setup1: TMenuItem;
    AddFiles1: TMenuItem;
    DeleteFile1: TMenuItem;
    Clear1: TMenuItem;
    Build1: TMenuItem;
    N3: TMenuItem;
    Options1: TMenuItem;
    procedure ToolButton1Click(Sender: TObject);
    procedure ToolButton3Click(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ToolButton5Click(Sender: TObject);
    procedure ToolButton7Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure N1Click(Sender: TObject);
    procedure AddFiles1Click(Sender: TObject);
    procedure DeleteFile1Click(Sender: TObject);
    procedure Clear1Click(Sender: TObject);
    procedure Build1Click(Sender: TObject);
    procedure Options1Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    Filename: string;

    function  GetFileSize(Name: string): integer;
    procedure AddFiles;
    procedure BuildSetup;
    procedure SaveSetup(Name: string);
    procedure OpenSetup(Name: string);
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.ToolButton1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.ToolButton3Click(Sender: TObject);
begin
  if lvFiles.Selected <> Nil then
    lvFiles.Items.Delete(lvFiles.Selected.Index);
end;

procedure TfrmMain.ToolButton2Click(Sender: TObject);
begin
  AddFiles;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'CreateSetup.ini');
  try
    edSetupFile.Text := Ini.ReadString('Setup','LastExe',ExtractFilePath(Application.ExeName) + 'setup.exe');
    Filename := Ini.ReadString('Setup','LastSetup','');
    if Filename <> '' then
      OpenSetup(Filename);
  finally
    Ini.Free;
  end;
end;

procedure TfrmMain.ToolButton5Click(Sender: TObject);
begin
  BuildSetup;
end;

function TfrmMain.GetFileSize(Name: string): integer;
var
   f: file of Byte;
begin
  AssignFile(f,Name);
  Reset(f);
  Result := FileSize(f);
  CloseFile(f);
end;

procedure TfrmMain.AddFiles;
var
  i,Sz: integer;
begin
  dlgOpen.Options := dlgOpen.Options + [ofAllowMultiSelect];
  dlgOpen.Filter := 'All Files (*.*)|*.*';
  if dlgOpen.Execute then begin
    for i := 0 to dlgOpen.Files.Count - 1 do begin
      with lvFiles.Items.Add do begin
        Caption := '';
        SubItems.Add(dlgOpen.Files[i]);
        Sz := GetFileSize(dlgOpen.Files[i]);
        SubItems.Add('');
                                    // Sz / 1 = Convert to float.
        SubItems.AddObject(Format('%.0n',[Sz / 1]),TObject(Sz));
        SubItems.Add('');
      end;
    end;
  end;
end;

procedure TfrmMain.BuildSetup;
var
  i: integer;
  Stream: TFileStream;
  FD: TFileData;
  S: string;

procedure AddSetupFile(Name: string; FileSize: integer);
var
  Sz: integer;
  RStream: TFileStream;
  Buf: array[0..1023] of byte;
begin
  lblResult.Caption := Name;
  lblResult.Visible := True;
  Progress.Position := 0;
  Progress.Max := FileSize;
  Progress.Visible := True;
  try
    RStream := TFileStream.Create(Name,fmOpenRead);
    repeat
      Sz := RStream.Read(Buf,SizeOf(Buf));
      Stream.Write(Buf,Sz);
      Progress.Position := Progress.Position + Sz;
      Application.ProcessMessages;
    until (Sz <> SizeOf(Buf));
  finally
    Progress.Visible := False;
    lblResult.Visible := False;
  end;
end;

begin
  S := ExtractFilePath(Application.ExeName) + '_setup.exe';
  CopyFile(PChar(S),PChar(edSetupFile.Text),False);
  Stream := TFileStream.Create(edSetupFile.Text,fmOpenWrite);
  try
    Stream.Seek(0,soFromEnd);
    for i := 0 to lvFiles.Items.Count - 1 do
      AddSetupFile(lvFiles.Items[i].SubItems[0],Integer(lvFiles.Items[i].SubItems.Objects[2]));
    for i := 0 to lvFiles.Items.Count - 1 do begin
      FillChar(FD,SizeOf(TFileData),0);

      S := ExtractFileName(lvFiles.Items[i].SubItems[0]);
      if lvFiles.Items[i].SubItems[3] <> '' then
        S := S + ':' + lvFiles.Items[i].SubItems[3];
      Move(Pointer(S)^,FD.Name[0],Length(S));

      S := lvFiles.Items[i].SubItems[1];
      Move(Pointer(S)^,FD.Params[0],Length(S));

      FD.Size := Integer(lvFiles.Items[i].SubItems.Objects[2]);
      if lvFiles.Items[i].Checked then
        FD.Flags := FD_FLAG_EXECUTE;
      Stream.Write(FD,SizeOf(TFileData));
    end;
    i := lvFiles.Items.Count;
    Stream.Write(i,SizeOf(longword));
    i := SETUPEXE_ID;
    Stream.Write(i,SizeOf(longword));
  finally
    Stream.Free;
  end;
end;

procedure TfrmMain.ToolButton7Click(Sender: TObject);
var
  CmdLine,WaitFor: string;
begin
  if lvFiles.Selected <> Nil then begin
    CmdLine := lvFiles.Selected.SubItems[1];
    WaitFor := lvFiles.Selected.SubItems[3];
    if TfrmOptions.Create(Application).Execute(CmdLine,WaitFor) then begin
      lvFiles.Selected.SubItems[1] := CmdLine;
      lvFiles.Selected.SubItems[3] := WaitFor;
      if not lvFiles.Selected.Checked and ((CmdLine <> '') or (WaitFor <> '')) then
        lvFiles.Selected.Checked := True;
    end;
  end;
end;

procedure TfrmMain.Exit1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.SaveSetup(Name: string);
var
  i: integer;
  Lines: TStringList;
begin
  Lines := TStringList.Create;
  try
    for i := 0 to lvFiles.Items.Count - 1 do with lvFiles.Items[i] do
      Lines.Add(Format('%d(%s),(%s),(%s)',[Ord(Checked),SubItems[0],SubItems[1],SubItems[3]]));
    Lines.SaveToFile(Name);
    Filename := Name;
    Caption := 'Create Setup [' + Filename + ']';
  finally
    Lines.Free;
  end;
end;

procedure TfrmMain.OpenSetup(Name: string);
var
  i,Sz: integer;
  Lines: TStringList;
  Line,FName,Params,WaitFor: string;
  Exec: boolean;

function GetNextItem(var S: string): string;
var
  p1,p2: integer;
begin
  Result := '';
  p1 := Pos('(',S);
  p2 := Pos(')',S);
  if (p1 > 0) and (p2 > 0) and (p2 > p1) then begin
    Result := Copy(S,p1 + 1,p2 - p1 - 1);
    S := Copy(S,p2 + 1,1024);
  end
  else
    S := '';
end;

begin
  lvFiles.Items.Clear;
  Lines := TStringList.Create;
  try
    Lines.LoadFromFile(Name);
    for i := 0 to Lines.Count - 1 do begin
      FName := '';
      Line := Lines[i];
      Exec := Copy(Line,1,1) = '1';
      FName := GetNextItem(Line);
      Params := GetNextItem(Line);
      WaitFor := GetNextItem(Line);
      if FName <> '' then with lvFiles.Items.Add do begin
        Caption := '';
        Checked := Exec;
        SubItems.Add(FName);
        Sz := GetFileSize(FName);
        SubItems.Add(Params);
                                    // Sz / 1 = Convert to float.
        SubItems.AddObject(Format('%.0n',[Sz / 1]),TObject(Sz));
        SubItems.Add(WaitFor);
      end;
    end;
    Filename := Name;
    Caption := 'Create Setup [' + Filename + ']';
  finally
    Lines.Free;
  end;
end;

procedure TfrmMain.Save1Click(Sender: TObject);
begin
  dlgSave.Filename := Filename;
  if dlgSave.Execute then
    SaveSetup(dlgSave.Filename);
end;

procedure TfrmMain.N1Click(Sender: TObject);
begin
  dlgOpen.Options := dlgOpen.Options - [ofAllowMultiSelect];
  dlgOpen.Filter := 'Setup Files (*.set)|*.set|All Files (*.*)|*.*';
  dlgOpen.Filename := Filename;
  if dlgOpen.Execute then
    OpenSetup(dlgOpen.Filename);
end;

procedure TfrmMain.AddFiles1Click(Sender: TObject);
begin
  ToolButton2Click(Self);
end;

procedure TfrmMain.DeleteFile1Click(Sender: TObject);
begin
  ToolButton3Click(Self);
end;

procedure TfrmMain.Clear1Click(Sender: TObject);
begin
  lvFiles.Items.Clear;
end;

procedure TfrmMain.Build1Click(Sender: TObject);
begin
  ToolButton5Click(Self);
end;

procedure TfrmMain.Options1Click(Sender: TObject);
begin
  ToolButton7Click(Self);
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create(ExtractFilePath(Application.ExeName) + 'CreateSetup.ini');
  try
    Ini.WriteString('Setup','LastExe',edSetupFile.Text);
    Ini.WriteString('Setup','LastSetup',Filename);
  finally
    Ini.Free;
  end;
end;

end.
